local AmidRT = LibStub("AceAddon-3.0"):NewAddon("AmidRT", "AceConsole-3.0", "AceEvent-3.0")

local AceGUI = LibStub("AceGUI-3.0")
local AceTimer = LibStub("AceTimer-3.0")
local BB = LibStub("LibBabble-Boss-3.0"):GetUnstrictLookupTable()
local LDBIcon = LibStub("LibDBIcon-1.0", true)
local LDB = LibStub("LibDataBroker-1.1", true)

local selectedevent = nil -- Selected Event in Menu
local lastKill = nil -- Last Boss killed
local raidactive = nil -- If currently tracking
local divider = ("#"):rep(10) -- Divider for Announcement to Guildchat

local itemid = nil
local LinkBox = nil

Orig_ChatEdit_InsertLink = ChatEdit_InsertLink
function ChatEdit_InsertLink(text)
  if text and LinkBox and LinkBox:IsVisible() then
    LinkBox:SetText(text)
	itemid = text
    return true
  else
    Orig_ChatEdit_InsertLink(text)
  end
end

-- Those are ignored (Marks, Emblems, Gems, etc.)
local ignoredItems = {
	[40752] = true, -- Emblem of Heroism
	[40753] = true, -- Emblem of Valor
	[34057] = true -- Abyss Crystal
}

-- Ignored as well (Sartharion Adds)
local ignoredBosses = {
	[BB["Tenebron"]]= true,
	[BB["Shadron"]]= true,
	[BB["Vesperon"]]= true,
	[BB["Stalagg"]]= true,
	[BB["Feugen"]]= true,
	[BB["Death Knight Understudy"]]= true
}

-- MultiEncounters like 4Horsemen
local multiEncounters = {}

AmidRT.options = {
	desc = "Amid Raid Tracker",
	type = "group",
	args = {
		show = {
			name = "show",
			desc = "Show the AmidRT window",
			type = "execute",
			func = function()
				if AmidRT.frame and AmidRT.frame:IsShown() then
					AmidRT.frame:Hide();
				end
				AmidRT:ShowGUI()
			end,
		},
		announce = {
			name = "announce",
			desc = "Toggle Announcement to Guild on/off",
			type = "toggle",
			get = function(info) return AmidRT.db.profile.announce end,
			set = function(info,val) AmidRT.db.profile.announce = val AmidRT:Print("Announcement set to: "..tostring(val)) end,
		},
		clear = {
			name = "clear",
			desc = "Clear all data",
			type = "execute",
			func = function()
				AmidRT:ClearAll()
				if AmidRT.frame and AmidRT.frame:IsShown() then
					AmidRT.frame:Hide();
					AmidRT:ShowGUI()
				end
			end,
		},
		difficulty = {
			name = "difficulty",
			desc = "Needed dungeon difficulty. (Heroic = true, doesn't matter = false",
			type = "toggle",
			get = function(info) return AmidRT.db.profile.difficulty end,
			set = function(info,val) AmidRT.db.profile.difficulty = val AmidRT:Print("Difficulty set to: "..tostring(val)) end,
		},
		ersatz = {
			name = "ersatz",
			desc = "Toggle Ersatz-Raid",
			type = "toggle",
			get = function(info) return AmidRT.db.profile.ersatz end,
			set = function(info,val) AmidRT.db.profile.ersatz = val AmidRT:Print("Ersatz set to: "..tostring(val)) end,
		},
		members = {
			name = "members",
			desc = "Minimum Number of Raidmembers",
			type = "input",
			set = function(info,val)
						AmidRT.db.profile.members = tonumber(val)
						AmidRT:Print("Members set to: "..tostring(val))
						AmidRT:RAID_ROSTER_UPDATE()
			end
		},
		minimap = {
			name = "minimap",
			desc = "Toggle Minimap Icon",
			type = "toggle",
			get = function(info) return AmidRT.db.profile.minimap end,
			set = function(info,val)
						AmidRT.db.profile.minimap = val
						if val then
							LDBIcon:Show("AmidRT")
						else
							LDBIcon:Hide("AmidRT")
						end
			end,
		},
		popup = {
			name = "popup",
			desc = "Toggle DKP Popup",
			type = "toggle",
			get = function(info) return AmidRT.db.profile.popup end,
			set = function(info,val) AmidRT.db.profile.popup = val AmidRT:Print("Popup set to: "..tostring(val)) end,
		},
		standby = {
			type = "toggle",
			name = "Standby",
			desc = "Toggles AmidRT on/off",
			get = function(info) return AmidRT.db.profile.standby end,
			set = function(info, val)
				AmidRT.db.profile.standby = val
				if val then
					AmidRT:Disable()
				else
					AmidRT:Enable()					
				end
			end,
		},
	},
}

local defaults = {
	profile = {
		event = {},
		announce = false,
		popup = true,
		ersatz = false,
		difficulty = false,
		currentTrackset = nil, -- Last created Trackset
		currentAnnounceset = nil,
		generic = nil, -- Generic Event for current Zone (Trash loot, Attendance DKP)
		members = 11,
		standby = false,
		minimap = false,
		minimapIcon = {},
	}
}

function AmidRT:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("AmidRTDB", defaults, "profile")
	LibStub("AceConfig-3.0"):RegisterOptionsTable("AmidRT", self.options)
	LibStub("AceConfigCmd-3.0"):CreateChatCommand("amidrt", "AmidRT")
	
	if LDB then
		self.ldb = LDB:NewDataObject("AmidRT",
				{
					type =  "launcher", 
					label = "AmidRT\n"..
							"|cffffff00Left click|r to open the GUI.\n"..
							"|cffffff00Right click|r to clear tracked data.",
					icon = "Interface\\Addons\\AmidRT\\icon",
					OnClick = function(clickedframe, button)
						if button == "LeftButton" then
							if self.frame and self.frame:IsShown() then
								self.frame:Hide();
							end
							self:ShowGUI()
						elseif button == "RightButton" then
							self:ClearAll()
							if self.frame and self.frame:IsShown() then
								self.frame:Hide();
								self:ShowGUI()
							end
						end
					end,
				})
		if LDBIcon then
			LDBIcon:Register("AmidRT", self.ldb, self.db.profile.minimapIcon)
			if not self.db.profile.minimap then
				LDBIcon:Hide("AmidRT")
			end
		end
	end
	
	if self.db.profile.standby then
		self:Disable()
	else
		self:Enable()
	end
end

function AmidRT:OnEnable()
	self:RegisterEvent("RAID_ROSTER_UPDATE")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	self:RAID_ROSTER_UPDATE()
	self:ResetEncounters()
	self:Print("is now enabled")
end

function AmidRT:OnDisable()
	self:UnregisterAllEvents()
	raidactive = nil
	self.db.profile.currentTrackset = nil
	self.db.profile.generic = nil
	self:Print("is now disabled")
end

function AmidRT:ResetEncounters()
	multiEncounters = {}
end

function AmidRT:RAID_ROSTER_UPDATE()
	local num = GetNumRaidMembers()
	
	local id, inBG = false
	for i=1, MAX_BATTLEFIELD_QUEUES do
		_, _, id = GetBattlefieldStatus(i);
		if( id > 0 ) then
			inBG = true
		end
	end
	if not inBG and ((self.db.profile.difficulty and GetCurrentDungeonDifficulty() == 2) or not self.db.profile.difficulty) then
		-- Only check, if raid > 10 players
		if not raidactive and num >= self.db.profile.members then
			raidactive = true
			GuildRoster()
			self:ResetEncounters()
			self:Print("Tracking activated.")
			self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
			self:RegisterEvent("CHAT_MSG_LOOT")
			if not self.db.profile.generic then
				self:AddGeneric()
				self.db.profile.generic = self.db.profile.currentTrackset
			end
		elseif raidactive and num < self.db.profile.members then
			raidactive = nil
			self:Print("Tracking deactivated.")
			self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
			self:UnregisterEvent("CHAT_MSG_LOOT")
			self.db.profile.currentTrackset = nil
			self.db.profile.generic = nil
		elseif num >= self.db.profile.members then
			for i = 1, GetNumRaidMembers() do
				local name = GetRaidRosterInfo(i)
				local class = UnitClass("raid"..i)
				local race = UnitRace("raid"..i)
				local level = UnitLevel("raid"..i)
				local found = nil
				for i,v in pairs(self.db.profile.event[self.db.profile.generic].attendants) do
					if v.name == name then
						found = true
					end
				end
				if not found then
					table.insert(self.db.profile.event[self.db.profile.generic].attendants, {['name']=name, ['class']=class, ['race']=race, ['level']=level})
				end
			end
		end
	end
end

local function getGameTimeAsUnix()
	return time()
end

function AmidRT:FromUnixTime(timestamp)
   local dt = date("*t", timestamp)
   return string.format("%02d/%02d/%d %02d:%02d:%02d", dt.month, dt.day, dt.year, dt.hour, dt.min, dt.sec)
end

local function whisperHandler()
	local player = arg2

	local text = arg1:lower():trim():gsub("^%l", string.upper)
	local set = AmidRT.db.profile.event[AmidRT.db.profile.currentAnnounceset]
	if set then
		local found = nil
		local inGuild = nil
		local class = nil
		local level = nil
		for i, v in ipairs(set.attendants) do
			if v == text then
				found = true
				SendChatMessage(format("<AmidRT>: %q ist bereits eingetragen.",text), "WHISPER", nil, player)
				break
			end
		end
		if not found then
			for i=1,GetNumGuildMembers(true) do
				if text == GetGuildRosterInfo(i) then
					_,_,_,level,class = GetGuildRosterInfo(i)
					inGuild = true
					break
				end
			end
		end
		if not inGuild then
			SendChatMessage(format("<AmidRT>: %q ist kein Mitglied von 'Amid Reloaded'.",text), "WHISPER", nil, player)
		end
		if inGuild and not found then
			local set2 = AmidRT.db.profile.event[AmidRT.db.profile.generic]
			found = nil
			for i, v in ipairs(set2.attendants) do
				if v == text then
					found = true
					break
				end
			end
			if not found then
				table.insert(set2.attendants, {['name']=text, ['class']=class, ['level']=level})
			end
			table.insert(set.attendants, {['name']=text, ['class']=class, ['level']=level})
			SendChatMessage(format("<AmidRT>: %q wurde dem Bosskill hinzugef\195\188gt.",text), "WHISPER", nil, player)
			AmidRT:Print(format("%q hinzugef\195\188gt von %s.",text, player))
			return true
		end
	end
end

local function whisperInformHandler(msg)
	if msg:find("^<AmidRT>") then return true end
end

-- Get Raidmembers
function AmidRT:TakeAttendance()
	local newTrackset = {
		name = lastKill,
		zone = GetRealZoneText(),
		attendants = {},
		date = getGameTimeAsUnix(),
		loot = {},
	}

	for i = 1, GetNumRaidMembers() do
		local name,_,group = GetRaidRosterInfo(i)
		local class = UnitClass("raid"..i)
		local race = UnitRace("raid"..i)
		local level = UnitLevel("raid"..i)
		if group < 6 or not self.db.profile.announce then
			table.insert(newTrackset.attendants, {['name']=name, ['class']=class, ['race']=race, ['level']=level})
		end
	end
	table.insert(self.db.profile.event, newTrackset)
	self.db.profile.currentTrackset = #self.db.profile.event
	self.db.profile.currentAnnounceset = #self.db.profile.event
	
	if (GetNumRaidMembers() > 25 or self.db.profile.announce) and self.db.profile.ersatz then
	
		local newTrackset = {
			name = lastKill.." Ersatz",
			zone = GetRealZoneText(),
			attendants = {},
			date = getGameTimeAsUnix(),
			loot = {},
		}
		for i = 1, GetNumRaidMembers() do
			local name,_,group = GetRaidRosterInfo(i)
			local class = UnitClass("raid"..i)
			local race = UnitRace("raid"..i)
			local level = UnitLevel("raid"..i)
			if group > 5 then
				table.insert(newTrackset.attendants, {['name']=name, ['class']=class, ['race']=race, ['level']=level})
			end
		end
		table.insert(self.db.profile.event, newTrackset)
		self.db.profile.currentAnnounceset = #self.db.profile.event
	end

	if self.db.profile.announce then
		ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER", whisperHandler)
		ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER_INFORM", whisperInformHandler)

		local str = nil
		str = format("AmidRT: %s down! Ersatzb\195\164nkler mir den Namen ihres Mainchars whispern. Ihr habt 3 Minuten Zeit!",lastKill)
		SendChatMessage(divider, "GUILD")
		SendChatMessage(str, "GUILD")
		SendChatMessage(divider, "GUILD")

		AceTimer:ScheduleTimer(self.AttendanceTimeout, 180)
	end
end

function AmidRT:AttendanceTimeout()
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_WHISPER", whisperHandler)
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_WHISPER_INFORM", whisperInformHandler)

	local str = "AmidRT: Ersatzbankerfassung vorbei!"
	SendChatMessage(str, "GUILD")
end

function AmidRT:PLAYER_REGEN_ENABLED()
	local BigWigs = _G["BigWigs"]
	if not UnitIsFeignDeath("player") then
		local go = BigWigs.modulePrototype:Scan()
		if not go then
			self:ResetEncounters()
			return
		end
	end

	if not UnitAffectingCombat("player") then
		self:ScheduleEvent(self.PLAYER_REGEN_ENABLED, 2, self)
	end
end

local linkpat = '|c........|Hitem:(%%d+):.-|h|r'
local globalpatterns = {
   "LOOT_ITEM",
   "LOOT_ITEM_SELF",
}
local patterns = {}
for _, global in ipairs(globalpatterns) do
   local pattern = _G[global]
   if strsub(pattern, 1,2) == "%s" then
      pattern = "([^ ]+)"..strsub(pattern, 3)
   end
   patterns[global] = gsub(gsub(pattern, "%%d", "(%%d+)"), "%%s", linkpat)
end

-- Check for Loot
function AmidRT:CHAT_MSG_LOOT(_,msg)
   player, item = strmatch(msg, patterns["LOOT_ITEM"])
   if player and item then
      self:ItemLooted(player, item)
      return
   end

   item = strmatch(msg, patterns["LOOT_ITEM_SELF"])
   if item then
      self:ItemLooted(UnitName("player"), item)
      return
   end
end

-- Check for Bosskill
function AmidRT:COMBAT_LOG_EVENT_UNFILTERED(_, _, event, _, _, _, _, mob)
	local mE = nil
	if event ~= "UNIT_DIED" and event ~= "UNIT_DESTROYED" then return end
	if mob and not ignoredBosses[mob] and BB[mob] then
		for i,v in pairs(multiEncounters) do
			if v[mob] then
				v[mob] = false
				self:Print(format("%s died",mob))
				mE = i
			end
		end
		if mE then
			local clear = true
			for i,v in pairs(multiEncounters[mE]) do
				if v then
					clear = false
				end
			end
			if clear then
				self:Print(format("%s down. Erfasse Anwesenheit...",mE))
				lastKill = mE
				self:TakeAttendance()
			end
		else
			self:Print(format("%s down. Erfasse Anwesenheit...",mob))
			lastKill = mob
			self:TakeAttendance()
		end
	end
end

-- Clear all Data
function AmidRT:ClearAll()
	self.db.profile.currentTrackset = nil
	self.db.profile.generic = nil
	self.db.profile.event = {}
	lastKill = nil
	
	self:Print("All data cleared!")
end

-- Get Item and save into DB
function AmidRT:ItemLooted(player, item)
	local _, itemLink, itemRarity = GetItemInfo(item)
	
	local itemId = select(3, itemLink:find("item:(%d+):"))
	if not itemId then return false end
	itemId = tonumber(itemId:trim())
	if type(itemId) ~= "number" then return false end
	if ignoredItems[itemId] or (itemRarity < 4 and not ignoredItems[itemId]) then return false end
	
	table.insert(self.db.profile.event[self.db.profile.currentTrackset].loot, {
		player = player,
		item = itemId,
		dkp = 0,
	})
	
	if self.db.profile.popup then
		self:GetDKP()
	end
end

-- Convert selected kill to EQDKP-String
function AmidRT:XMLOutput(event)
	local text = "<RaidInfo>"
	local players_join = ""
	local players_leave = ""

	text = text .. "<key>" .. self:FromUnixTime(event.date) .. "</key>"
	text = text .. "<start>" .. self:FromUnixTime(event.date) .. "</start>"
	text = text .. "<end>" .. self:FromUnixTime(event.date+1) .. "</end>"
	text = text .. "<zone>" .. event.zone .. "</zone>"
	text = text .. "<note>" .. event.zone .. "(" ..  event.name .. ")</note>"

	text = text .. "<PlayerInfos>"

	for i, v in ipairs(event.attendants) do
		text = text .. "<key" .. i .. "><name>" .. v.name .. "</name>"
		if v.class then text = text .. "<class>" .. v.class .. "</class>" end
		if v.level then text = text .. "<level>" .. v.level .. "</level>" end
		if v.race then text = text .. "<race>" .. v.race .. "</race>" end
		text = text .. "</key" .. i .. ">"
		players_join = players_join .. "<key" .. i .. "><player>" .. v.name .. "</player><time>" .. self:FromUnixTime(event.date) .. "</time></key" .. i .. ">"
		players_leave = players_leave .. "<key" .. i .. "><player>" .. v.name .. "</player><time>" .. self:FromUnixTime(event.date+1) .. "</time></key" .. i .. ">"
	end

	text = text .. "</PlayerInfos>"

	text = text .. "<Bossevent>"
	
	text = text .. "<key" .. 1 .. "><name>" .. event.name .. "</name><time>" .. self:FromUnixTime(event.date) .. "</time><attendees/></key" .. 1 .. ">"

	text = text .. "</Bossevent>"

	text = text .. "<Join>" .. players_join .. "</Join>"
	text = text .. "<Leave>" .. players_leave .. "</Leave>"

	text = text .. "<Loot>"


	for i, item in ipairs(event.loot) do
		local itemName,_,rarity = GetItemInfo(item.item)
		local id = item.item
		local _,_,_,hex = GetItemQualityColor(rarity)
		text = text .. "<key" .. i .. ">"
		text = text .. "<ItemName>" .. (itemName or "?") .. "</ItemName>"
		text = text .. "<ItemID>" .. id .. "</ItemID>"
		text = text .. "<Color>" .. strsub(hex,3) .. "</Color>"
		text = text .. "<Count>1</Count>"
		text = text .. "<Player>" .. item.player .. "</Player>"
		text = text .. "<Note>" .. item.dkp .. " DKP</Note>"
		text = text .. "<Time>" .. self:FromUnixTime(event.date) .. "</Time>"
		text = text .. "<Zone>" .. event.zone .. "</Zone>"
		text = text .. "<Boss>" .. event.name .. "</Boss>"
		text = text .. "</key" .. i .. ">"
	end

	text = text .. "</Loot>"
	text = text .. "</RaidInfo>"
	return text
end

function AmidRT:SelectItem(widget, event, value)
	widget:ReleaseChildren()
	
	local sf = AceGUI:Create("ScrollFrame")
	sf:SetLayout("List")
	
	local e1 = AceGUI:Create("EditBox")
	e1:SetLabel("ItemID")
	e1:SetText(self.db.profile.event[selectedevent].loot[value].item)
	e1:SetCallback("OnEnterPressed",function(widget,event,text) self.db.profile.event[selectedevent].loot[value].item = strtrim(text) end )
	
	local e2 = AceGUI:Create("EditBox")
	e2:SetLabel("Looted by")
	e2:SetText(self.db.profile.event[selectedevent].loot[value].player)
	e2:SetCallback("OnEnterPressed",function(widget,event,text) self.db.profile.event[selectedevent].loot[value].player = strtrim(text) end )
	
	local b1 = AceGUI:Create("Button")
	b1:SetText("D")
	b1:SetWidth(40)
	b1:SetCallback("OnClick", function()
								self.db.profile.event[selectedevent].loot[value].player = "disenchanted"
								e2:SetText("disenchanted")
							end )
	local b2 = AceGUI:Create("Button")
	b2:SetText("B")
	b2:SetWidth(40)
	b2:SetCallback("OnClick", function()
								self.db.profile.event[selectedevent].loot[value].player = "bank"
								e2:SetText("bank")
							end )
	local b3 = AceGUI:Create("Button")
	b3:SetText("T")
	b3:SetWidth(40)
	b3:SetCallback("OnClick", function()
								self.db.profile.event[selectedevent].loot[value].player = UnitName("target")
								e2:SetText(UnitName("target"))
							end )
	
	local e3 = AceGUI:Create("EditBox")
	e3:SetLabel("DKP")
	e3:SetText(self.db.profile.event[selectedevent].loot[value].dkp)
	e3:SetCallback("OnEnterPressed",function(widget,event,text) self.db.profile.event[selectedevent].loot[value].dkp = strtrim(text) end )
	
	local b4 = AceGUI:Create("Button")
	b4:SetText("Item entfernen")
	b4:SetCallback("OnClick", function()
								table.remove(self.db.profile.event[selectedevent].loot, value)
								e1:SetText("")
								e2:SetText("")
								e3:SetText("")
								
								local tree = {}
								for i,v in ipairs(self.db.profile.event[selectedevent].loot) do 
									table.insert(tree, {value = i, text = GetItemInfo(v.item)})
								end
								
								widget:SetTree(tree)
							end )
	sf:AddChild(e1)
	sf:AddChild(e2)
	sf:AddChild(b1)
	sf:AddChild(b2)
	sf:AddChild(b3)
	sf:AddChild(e3)
	sf:AddChild(b4)
	widget:AddChild(sf)
end
	
	
function AmidRT:SelectEvent(widget, event, value)
	widget:ReleaseChildren()
	selectedevent = value
	
	local sf = AceGUI:Create("ScrollFrame")
	sf:SetLayout("Flow")
	
	local h1 = AceGUI:Create("Heading")
	h1:SetText("Event")
	h1.width = "fill"
	
	local sg0 = AceGUI:Create("SimpleGroup")
	sg0:SetWidth(200)
	local sg1 = AceGUI:Create("SimpleGroup")
	sg1:SetLayout("Flow")
	local e1 = AceGUI:Create("EditBox")
	e1:SetLabel("Zone")
	e1:SetWidth(150)
	e1:SetText(self.db.profile.event[value].zone)
	e1:SetCallback("OnEnterPressed",function(widget,event,text) self.db.profile.event[value].zone = strtrim(text) end )
	local b1 = AceGUI:Create("Button")
	b1:SetText("T")
	b1:SetWidth(40)
	b1:SetCallback("OnClick", function()
								self.db.profile.event[value].zone = GetRealZoneText()
								e1:SetText(GetRealZoneText())
							end )
	
	local sg2 = AceGUI:Create("SimpleGroup")
	sg2:SetWidth(200)
	local me1 = AceGUI:Create("MultiLineEditBox")
	me1:SetLabel("Attendants")
	local attendees = ""
	for i,v in ipairs(self.db.profile.event[value].attendants) do
		attendees = attendees .. v.name .. "\n"
	end
	me1:SetText(attendees)
	me1:SetHeight(200)
	me1:SetWidth(200)
	me1:SetCallback("OnEnterPressed",function(widget,event,text)
										--TODO
									end )
	local b3 = AceGUI:Create("Button")
	b3:SetText("T")
	b3:SetWidth(40)
	b3:SetCallback("OnClick", function()
									local name, _ = UnitName("target")
									table.insert(self.db.profile.event[value].attendants, name)
									for i,v in ipairs(self.db.profile.event[value].attendants) do
										attendees = attendees .. v .. "\n"
									end
									me1:SetText(attendees)
							end )
							
	local sg3 = AceGUI:Create("SimpleGroup")
	sg3:SetLayout("Flow")
	local e2 = AceGUI:Create("EditBox")
	e2:SetLabel("Boss")
	e2:SetWidth(150)
	e2:SetText(self.db.profile.event[value].name)
	e2:SetCallback("OnEnterPressed",function(widget,event,text) self.db.profile.event[value].name = strtrim(text) end )
	local b2 = AceGUI:Create("Button")
	b2:SetText("T")
	b2:SetWidth(40)
	b2:SetCallback("OnClick", function()
								self.db.profile.event[value].zone = (UnitName("target") or GetRealZoneText())
								e2:SetText((UnitName("target") or GetRealZoneText()))
							end )
							
	local e3 = AceGUI:Create("EditBox")
	e3:SetLabel("Date")
	e3:SetWidth(150)
	e3:SetText(self.db.profile.event[value].date)
	e3:SetCallback("OnEnterPressed",function(widget,event,text) self.db.profile.event[value].date = strtrim(text) end )
		
	local h2 = AceGUI:Create("Heading")
	h2:SetText("Loot")
	h2.width = "fill"
	
	local t = AceGUI:Create("TreeGroup")
	t.width = "fill"
	t:SetHeight(300)
	t:SetLayout("Fill")
	
	local tree = {}
	for i,v in ipairs(self.db.profile.event[value].loot) do 
		table.insert(tree, {value = i, text = GetItemInfo(v.item)})
	end
	t:SetTree(tree)
	t:SetCallback("OnGroupSelected", function(w,e,v) self:SelectItem(w,e,v) end )

	local b6 = AceGUI:Create("Button")
	b6:SetWidth(150)
	b6:SetText("Item hinzuf\195\188gen")
	b6:SetCallback("OnClick", function()
								_,itemid = GetItemInfo(itemid)
								if(itemid) then
									_, _, itemid = string.find(itemid, "^|c%x+|%a+:(%d+):.+|h%[.*%]")
									table.insert(self.db.profile.event[selectedevent].loot, {['item']=itemid,['player']="player",['dkp']=0})
									
									local tree = {}
									for i,v in ipairs(self.db.profile.event[selectedevent].loot) do 
										table.insert(tree, {value = i, text = GetItemInfo(v.item)})
									end
									LinkBox:SetText("")
									
									t:SetTree(tree)
								else
									self:Print("Invalid Item: "..itemid)
								end
							end )

	local e4 = AceGUI:Create("EditBox")
	e4:SetLabel("Item")
	e4:SetWidth(150)
	e4:SetText("")
	e4:SetCallback("OnEnterPressed",function(widget,event,text) itemid = text end )
	LinkBox = e4
	
	local h3 = AceGUI:Create("Heading")
	h3:SetText("Misc")
	h3.width = "fill"
	
	local me2 = AceGUI:Create("MultiLineEditBox")
	me2:SetLabel("EQDKP-String")
	me2:SetText("")
	me2.width = "fill"
	me2:SetHeight(200)
	
	local b4 = AceGUI:Create("Button")
	b4:SetText("XML Output")
	b4:SetWidth(200)
	b4:SetCallback("OnClick", function() me2:SetText(self:XMLOutput(self.db.profile.event[value])) end )
	
	local b5 = AceGUI:Create("Button")
	b5:SetText("Kill entfernen")
	b5:SetWidth(200)
	b5:SetCallback("OnClick", function()
								table.remove(self.db.profile.event, value)
								self.db.profile.currentTrackset = #self.db.profile.event;
								widget.parent:Hide()
								self:ShowGUI()
							end )
	
	sf:AddChild(h1)
	sg1:AddChild(e1)
	sg1:AddChild(b1)
	sg3:AddChild(e2)
	sg3:AddChild(b2)
	sg2:AddChild(me1)
	sg2:AddChild(b3)
	sg0:AddChild(sg1)
	sg0:AddChild(sg3)
	sg0:AddChild(e3)
	sf:AddChild(sg0)
	sf:AddChild(sg2)
	sf:AddChild(h2)
	sf:AddChild(t)
	sf:AddChild(b6)
	sf:AddChild(e4)
	sf:AddChild(h3)
	sf:AddChild(b4)
	sf:AddChild(b5)
	sf:AddChild(me2)
	widget:AddChild(sf)
end

-- Create Menu
function AmidRT:ShowGUI()
	-- The Big Black One
	local f = AceGUI:Create("Frame")
	f:SetTitle("AmidRT Menu")
	f:SetLayout("Fill")
	
	self.frame = f
	
	local t = AceGUI:Create("TreeGroup")
	t:SetLayout("Fill")
	local tree = {}
	for i,v in ipairs(self.db.profile.event) do 
		table.insert(tree, {value = i, text = v.name})
	end
	f:SetStatusText(#tree .." entries found.")
	t:SetTree(tree)
	t:SetCallback("OnGroupSelected", function(w,e,v) self:SelectEvent(w,e,v) end )
	f:AddChild(t)
end

-- Adds generic Event for current Zone
function AmidRT:AddGeneric()
	local newTrackset = {
		name = "Anwesenheit",
		zone = "Generic",
		attendants = {},
		date = getGameTimeAsUnix(),
		loot = {},
	}
	for i = 1, GetNumRaidMembers() do
		local name = GetRaidRosterInfo(i)
		local class = UnitClass("raid"..i)
		local race = UnitRace("raid"..i)
		local level = UnitLevel("raid"..i)
		table.insert(newTrackset.attendants, {['name']=name, ['class']=class, ['race']=race, ['level']=level})
	end
	table.insert(self.db.profile.event, newTrackset)
	self.db.profile.generic = #self.db.profile.event
	self.db.profile.currentTrackset = #self.db.profile.event
end

function AmidRT:GetDKP()
	local lootid = #self.db.profile.event[self.db.profile.currentTrackset].loot
	local item = GetItemInfo(self.db.profile.event[self.db.profile.currentTrackset].loot[lootid].item)
	local player = self.db.profile.event[self.db.profile.currentTrackset].loot[lootid].player
	
	local f = AceGUI:Create("Frame")
	f:SetTitle("DKP eingeben")
	f:SetLayout("List")
	f:SetWidth(400)
	f:SetHeight(200)
	
	local sg = AceGUI:Create("SimpleGroup")
	
	local l1 = AceGUI:Create("Label")
	l1:SetWidth(300)
	l1:SetText(player..": "..item)
	
	local e = AceGUI:Create("EditBox")
	e:SetLabel("DKP:")
	e:SetWidth(300)
	e:SetText("0")
	e:SetCallback("OnEnterPressed",function(widget,event,text)
										self.db.profile.event[self.db.profile.currentTrackset].loot[lootid].dkp = strtrim(text)
									end )
	
	local b1 = AceGUI:Create("Button")
	b1:SetText("disenchanted")
	b1:SetWidth(200)
	b1:SetCallback("OnClick", function()
								self.db.profile.event[self.db.profile.currentTrackset].loot[lootid].player = "disenchanted"
								l1:SetText("disenchanted: "..item)
							end )
	local b2 = AceGUI:Create("Button")
	b2:SetText("bank")
	b2:SetWidth(200)
	b2:SetCallback("OnClick", function()
								self.db.profile.event[self.db.profile.currentTrackset].loot[lootid].player = "bank"
								l1:SetText("bank: "..item)
							end )
	
	sg:AddChild(l1)
	sg:AddChild(e)
	sg:AddChild(b1)
	sg:AddChild(b2)
	f:AddChild(sg)
end
